--  ----------------------------------------------------------------------------
-- FILENAME:      oracle_install.sql                                        --
-- VERSION:       1                                                           --
-- DATABASE:      Oracle 8i,9i,10g or later                                        --
-- SUMMARY:       Creates database objects necessary for NAM Risk Management.       --
-- INSTRUCTIONS:  Execute as SYSTEM/MANAGER or some other database            --
--                administrator.                                                   --
-- DISPLAY:       monospaced font, 80 columns                                 --
--  ----------------------------------------------------------------------------


SET TERMOUT ON;
SET ECHO OFF;


--------------------------------------------------------------------------------
-- CREATE NAM'S USER ACCOUNT                                               --
--                                                                            --
-- Create the database account used by the NAM.  In practice, this account --
-- should be used exclusively by the NAM.                                  --
--------------------------------------------------------------------------------


CONNECT SYSTEM/novell;

-- NOTE:
-- A schema is implicitly created when user is created.
-- GRANT RESOURCE to avoid error ORA-01950 (no privileges on tablespace).
CREATE USER netiq_risk IDENTIFIED BY novell DEFAULT TABLESPACE USERS;
ALTER USER netiq_risk quota unlimited on USERS;
--GRANT CONNECT TO netiq_risk;


--CONNECT netiq_risk/novell;

--  ----------------------------------------------------------------------------
--      CREATE TABLES                                                         --
--                                                                            --
--  ----------------------------------------------------------------------------

CREATE TABLE netiq_risk.usr
(
  id         VARCHAR2(100)  NOT NULL,
  uname       VARCHAR2(100),
  dn       VARCHAR2(500),
  
  CONSTRAINT pk_usr_id PRIMARY KEY (id)
);

CREATE TABLE netiq_risk.region
(
  rid         INTEGER                 NOT NULL,
  rname       VARCHAR2(100),
  rcode       VARCHAR(32),
  
  CONSTRAINT pk_region_rid PRIMARY KEY (rid)
);

CREATE TABLE netiq_risk.geo
(
  gid         INTEGER                 NOT NULL,
  gname       VARCHAR2(100),
  rid         INTEGER,
  
  CONSTRAINT pk_geo_gid PRIMARY KEY (gid),
  CONSTRAINT fk_geo_rid FOREIGN KEY (rid) REFERENCES netiq_risk.region(rid)
);

CREATE TABLE netiq_risk.country
(
  cid         INTEGER                 NOT NULL,
  cname       VARCHAR2(100),
  rid         INTEGER,
  ccode       VARCHAR(32),
  
  CONSTRAINT pk_country_cid PRIMARY KEY (cid),
  CONSTRAINT fk_country_rid FOREIGN KEY (rid) REFERENCES netiq_risk.region(rid)
);

CREATE TABLE netiq_risk.state
(
  sid         INTEGER                 NOT NULL,
  sname       VARCHAR(100),
  scode       VARCHAR(32),
  cid         INTEGER,
  
  CONSTRAINT pk_state_sid PRIMARY KEY (sid),
  CONSTRAINT fk_state_sid FOREIGN KEY (sid) REFERENCES netiq_risk.country(cid)
);

CREATE TABLE netiq_risk.city
(
  citid         INTEGER                 NOT NULL,
  citname       VARCHAR2(100),
  cid           INTEGER,
  
  CONSTRAINT pk_city_citid PRIMARY KEY (citid),
  CONSTRAINT fk_city_cid FOREIGN KEY (cid) REFERENCES netiq_risk.country(cid)
);

CREATE TABLE netiq_risk.transaction
(
  uuid         INTEGER                 NOT NULL,
  id          VARCHAR2(100)  NOT NULL,
  displayname VARCHAR2(32),
  riskscore INTEGER,
  lastlogintime INTEGER,
  city      INTEGER,
  country   INTEGER,
  geolocation INTEGER,
  state  INTEGER,
  region INTEGER,
  ip VARCHAR(40),
  loginresult       CHAR(1),
  riskcategory VARCHAR2(10),
  createddate DATE,
  createdby VARCHAR2(32),
  modifiedby VARCHAR2(32),
  modifieddate DATE,
  twofactorauthresult CHAR(1),
  postcode VARCHAR(32),
  regioncode VARCHAR(32),
  metrocode VARCHAR(32),
  areacode VARCHAR(32),
  deviceid VARCHAR(32),
  errormsg VARCHAR(200),
  timeZone VARCHAR(50),
  organization VARCHAR(100),
  anonymous  CHAR(1), 
  privateipaddress  CHAR(1), 

  CONSTRAINT pk_transaction_uuid PRIMARY KEY (uuid),
  CONSTRAINT fk_transction_id FOREIGN KEY (id) REFERENCES netiq_risk.usr(id)
);

CREATE TABLE netiq_risk.extra
(
  id         VARCHAR2(32)  NOT NULL,
  custom_string_entry1       VARCHAR2(100),
  custom_int_entry2      INTEGER,
  custom_char_entry3       CHAR(1),
  
  CONSTRAINT fk_extra_id FOREIGN KEY (id) REFERENCES netiq_risk.usr(id)
);

CREATE TABLE netiq_risk.version
(
  prodname       VARCHAR2(100),
  prodver       VARCHAR2(100)
);

--  ----------------------------------------------------------------------------
--      CREATE Index                                                          --
--                                                                            --
--  ----------------------------------------------------------------------------

CREATE INDEX netiq_risk.idx_usr_name ON netiq_risk.usr(uname);
CREATE INDEX netiq_risk.idx_region_rname ON netiq_risk.region(rname);
CREATE INDEX netiq_risk.idx_geo_gname ON netiq_risk.geo(gname);
CREATE INDEX netiq_risk.idx_geo_rid ON netiq_risk.geo(rid);
CREATE INDEX netiq_risk.idx_country_cname ON netiq_risk.country(cname);
CREATE INDEX netiq_risk.idx_country_rid ON netiq_risk.country(rid);
CREATE INDEX netiq_risk.idx_city_citname ON netiq_risk.city(citname);
CREATE INDEX netiq_risk.idx_city_cid ON netiq_risk.city(cid);
CREATE INDEX netiq_risk.idx_transaction_id ON netiq_risk.transaction(id);
CREATE INDEX netiq_risk.idx_transaction_match_ip ON netiq_risk.transaction(id, ip);
CREATE INDEX netiq_risk.idx_transaction_match_city ON netiq_risk.transaction(id, city);
CREATE INDEX netiq_risk.idx_transaction_match_country ON netiq_risk.transaction(id, country);
CREATE INDEX netiq_risk.idx_transaction_match_geo ON netiq_risk.transaction(id, geolocation);
CREATE INDEX netiq_risk.idx_transaction_match_region ON netiq_risk.transaction(id, region);
CREATE INDEX netiq_risk.idx_transaction_match_state ON netiq_risk.transaction(id, state);
CREATE INDEX netiq_risk.idx_version_prodname ON netiq_risk.version(prodname);

--  ----------------------------------------------------------------------------
--      CREATE Sequence                                                        --
--                                                                            --
--  ----------------------------------------------------------------------------

CREATE SEQUENCE netiq_risk.seq_region_rid
  START WITH 1
  INCREMENT BY 1
  NOMINVALUE
  NOMAXVALUE
  CACHE 128
  ORDER;

 CREATE SEQUENCE netiq_risk.seq_geo_gid
  START WITH 1
  INCREMENT BY 1
  NOMINVALUE
  NOMAXVALUE
  CACHE 128
  ORDER;

 CREATE SEQUENCE netiq_risk.seq_country_cid
  START WITH 1
  INCREMENT BY 1
  NOMINVALUE
  NOMAXVALUE
  CACHE 128
  ORDER;

  CREATE SEQUENCE netiq_risk.seq_state_sid
  START WITH 1
  INCREMENT BY 1
  NOMINVALUE
  NOMAXVALUE
  CACHE 128
  ORDER;

 CREATE SEQUENCE netiq_risk.seq_city_citid
  START WITH 1
  INCREMENT BY 1
  NOMINVALUE
  NOMAXVALUE
  CACHE 128
  ORDER;

 CREATE SEQUENCE netiq_risk.seq_transaction_uuid
  START WITH 1
  INCREMENT BY 1
  NOMINVALUE
  NOMAXVALUE
  CACHE 128
  ORDER;

--  ----------------------------------------------------------------------------
--      prepopulate static data                                               --
--                                                                            --
--  ----------------------------------------------------------------------------

insert into netiq_risk.region(rid,rname) values (netiq_risk.seq_region_rid.nextval,'Unknown');
insert into netiq_risk.geo(gid,gname,rid) values (netiq_risk.seq_geo_gid.nextval,'Unknown',1);
insert into netiq_risk.country(cid,cname,rid) values (netiq_risk.seq_country_cid.nextval,'Unknown',1);
insert into netiq_risk.state(sid,sname,cid) values (netiq_risk.seq_state_sid.nextval,'Unknown',1);
insert into netiq_risk.city(citid,citname,cid) values (netiq_risk.seq_city_citid.nextval,'Unknown',1);
commit;


